@@ -5,6 +5,7 @@ import numpy as np
 from scipy.stats import pearsonr
 import os
 from matplotlib.backends.backend_pdf import PdfPages
+import matplotlib.ticker as ticker
 import seaborn as sns
 import matplotlib.pyplot as plt
 import pysam
@@ -252,9 +253,6 @@ def RUST(df, offset, background, fasta_d):
     return e, f
 
 
-
-
-
 def generate_bed_df(args, info, fasta_d):
     """
     Reads in the bed file and info file, either choosing the biggest (monosome) file
@@ -296,15 +294,15 @@ def generate_bed_df(args, info, fasta_d):
         biggest_sample = max(read_nos, key=read_nos.get)
         print("Biggest sample is " + biggest_sample)
 
-        df, total_counts, total_counts_cds = read_bed_ish_file(filenames[biggest_sample], info, args,
+        df, total_counts, total_counts_cds, ss_df = read_bed_ish_file(filenames[biggest_sample], info, args,
                                                                fasta_d)
 
     else:  # it's a single sample
         print("Reading single sample")
-        df, total_counts, total_counts_cds = read_bed_ish_file(sample_dir + args.samples, info, args,
+        df, total_counts, total_counts_cds, ss_df = read_bed_ish_file(sample_dir + args.samples, info, args,
                                                                fasta_d)
 
-    return df, total_counts, total_counts_cds
+    return df, total_counts, total_counts_cds, ss_df
 
 
 def find_APE_of_reference(args, df, fasta_d, total_counts_cds):
@@ -529,14 +527,43 @@ def read_bed_ish_file(filename, info, args, fasta_d):
     df = df.merge(size_df, on=["transcript_id", "read_type", "end"])
     df = df.drop_duplicates().reset_index()
 
-    # Filter for abundant read types that are periodic
     total_counts = sum(df["counts_this_type_this_tx_this_pos"])
     total_counts_cds = sum(cds["counts_this_type_cds"])
+
+    # Get start and stop codon dfs for each read length
+    ss_df = make_ss_df(df)
+
+    # Filter for abundant read types that are periodic
     df = filter_non_periodic(df, args)
     df = df[~df["read_type"].isin(args.list_to_ignore)]
 
     assert len(df) > 0, "Filtering removed all reads! Trying setting less stringent parameters."
-    return df, total_counts, total_counts_cds
+    return df, total_counts, total_counts_cds, ss_df
+
+
+def gen_ss_df(df, cds_start_or_end, region):
+    """
+    Looks for number of reads at each distance from start or stop codon
+    cds_start_or_end = "cds_start" or "cds_end"
+    """
+    df['distance'] = df['end'] - df[cds_start_or_end]
+    df = df[abs(df['distance']) <= region]
+    df = df[['distance', 'read_length', "counts_this_type_this_tx_this_pos"]]
+    df = df.reset_index().groupby(['distance', 'read_length']).sum().drop_duplicates().reset_index()
+    df["frame"] = df["distance"] % 3
+    return df
+
+
+def make_ss_df(df, region=100):
+    """
+    This is a function for showing periodicity around start and stop codons
+    """
+    start_df = gen_ss_df(df, "cds_start", region)
+    start_df['start_or_end'] = "start"
+    end_df = gen_ss_df(df, "cds_end", region)
+    end_df["start_or_end"] = "end"
+    combined = start_df.append(end_df)
+    return combined.reset_index()
 
 
 def filter_non_periodic(df, args):
@@ -675,7 +702,7 @@ def find_best_offsets(these_ratios, this_type, args, ref_APEs):
     for key, value in r_values.items():
         if (int(this_type.split('_')[1]) + key) % 3 != 0 and not args.allow_out_of_frame:
             continue
-        if value >= args.max_A_offset and value <= args.min_A_offset:
+        if args.max_A_offset <= value <= args.min_A_offset:
             r_values_filtered[key] = value
     best_nt_offset = max(r_values_filtered, key=r_values.get)
 
@@ -855,7 +882,7 @@ def append_to_ref(ref_APEs, these_ratios, offset, this_type):
 
 
 def convert_bed_to_single_codon_res(args, df, fasta_d, ref_APEs, precalculated_offsets,
-                                    total_counts_cds, info, orf_df, oof_df, oof_distance=500):
+                                    total_counts_cds, info, orf_df, oof_df, ss_df, oof_distance=500):
     """
     This function finds the best APE offset for each
     read length that passes filtering by comparing to the reference APE RUST
@@ -985,7 +1012,7 @@ def convert_bed_to_single_codon_res(args, df, fasta_d, ref_APEs, precalculated_o
             kl_d_d[this_type] = save_KL(fractions_d[this_type],
                                         background_d[this_type],
                                         this_type, best_nt_offsets[this_type], args,
-                                        save=args.save_KL)
+                                        save=args.save_stats)
 
         if len(done_dict) == 0:
             break
@@ -1011,7 +1038,7 @@ def convert_bed_to_single_codon_res(args, df, fasta_d, ref_APEs, precalculated_o
 
     check_offsets_consistent(best_nt_offsets)
     write_files(df, full_d, best_nt_offsets, args, all_r_values, oof_df, oof_d, frameness_d, fasta_d, counts_df,
-                total_counts_cds, kl_d_d)
+                total_counts_cds, kl_d_d, ss_df)
 
     return best_nt_offsets
 
@@ -1094,7 +1121,7 @@ def order_d(d):
     return to_return
 
 
-def plot_KL_and_r(kl_d_d, pdf_pages, title, vmax=1, square=False, only_in_frame=False):
+def plot_KL_and_r(kl_d_d, pdf_pages, title, vmax=1.0, square=False, only_in_frame=False):
 
     read_types = []
     positions = []
@@ -1122,9 +1149,21 @@ def plot_KL_and_r(kl_d_d, pdf_pages, title, vmax=1, square=False, only_in_frame=
     pdf_pages.savefig(p.figure)
 
 
+def order_list(types):
+    """
+    wrapper around order_d()
+    """
+    d = {}
+    for this_type in types:
+        d[this_type] = 0
+
+    return order_d(d).keys()
+
+
 def plot_cds_metaprofile(df, best_nt_offsets, pdf_pages, min_abundance):
     df = df[df["counts_this_type_cds"] > min_abundance]
     types = df["read_type"].drop_duplicates()
+    types = order_list(types)
     for this_type in types:
         plt.clf()
         cds_metaprofile(df[df["read_type"] == this_type], best_nt_offsets, pdf_pages, this_type)
@@ -1266,8 +1305,28 @@ def plot_oof_heatmap(df, pdf_pages, window):
     pdf_pages.savefig(p2.figure)
 
 
+def plot_around_start_or_stop(ss_df, best_nt_offsets, pdf_pages):
+    # find which read lengths are of interest
+    lengths = []
+    for read_type in best_nt_offsets.keys():
+        read_length = int(read_type.split("_")[0])
+        if read_length not in lengths:
+            lengths.append(read_length)
+    lengths.sort()
+    for start_or_end in ['start', 'end']:
+        for this_length in lengths:
+            this_ss_df = ss_df[ss_df["start_or_end"] == start_or_end]
+            this_ss_df = this_ss_df[this_ss_df["read_length"] == this_length]
+            this_ss_df["counts"] = this_ss_df["counts_this_type_this_tx_this_pos"]
+            plt.clf()
+            p = sns.barplot(x="distance", y="counts", hue="frame", data=this_ss_df, dodge=False)
+            p.set_title(str(this_length) + 'nt reads around ' + start_or_end)
+            p.xaxis.set_major_locator(ticker.MultipleLocator(base=20))
+            pdf_pages.savefig(p.figure)
+
+
 def write_files(df, full_d, best_nt_offsets, args, r_values, oof_df, oof_d, frameness_d, fasta_d, counts_df,
-                total_counts_cds, kl_d_d):
+                total_counts_cds, kl_d_d, ss_df):
     print("Writing files...")
 
     df2 = df[df["read_type"].isin(best_nt_offsets.keys())]
@@ -1276,6 +1335,7 @@ def write_files(df, full_d, best_nt_offsets, args, r_values, oof_df, oof_d, fram
     if args.plot_graphs:
         with PdfPages(args.output_directory + args.output + "_plots.pdf") as pdf_pages:
             print("Saving graphs")
+            plt.clf()
             plot_abundances(counts_df, total_counts_cds, best_nt_offsets, pdf_pages, args.min_abundance)
             plt.clf()
             print("Saving KL heatmap")
@@ -1293,6 +1353,8 @@ def write_files(df, full_d, best_nt_offsets, args, r_values, oof_df, oof_d, fram
             print("Saving metaprofiles")
             plot_cds_metaprofile(df, best_nt_offsets, pdf_pages, args.min_abundance)
             plt.clf()
+            plot_around_start_or_stop(ss_df, best_nt_offsets, pdf_pages)
+            plt.clf()
             print("Saving PCA")
             plot_pca(full_d, best_nt_offsets, pdf_pages)
             plt.clf()
@@ -1300,9 +1362,6 @@ def write_files(df, full_d, best_nt_offsets, args, r_values, oof_df, oof_d, fram
                 print("Saving oof heatmap")
                 plot_oof_heatmap(oof_df, pdf_pages, args.oof_smooth_window)
 
-    if args.write_rust:
-        write_rust(full_d, best_nt_offsets, args)
-
     if args.keep_read_types_distinct:
         write_combined_distinct_bedgraph(df, best_nt_offsets, args)
 
@@ -1311,6 +1370,7 @@ def write_files(df, full_d, best_nt_offsets, args, r_values, oof_df, oof_d, fram
 
     if args.save_stats:
         save_stats(r_values, args, oof_d, frameness_d, df)
+        write_rust(full_d, best_nt_offsets, args)
 
 
     print("Complete!")
@@ -1375,7 +1435,7 @@ def write_combined_bedgraph(df, best_nt_offsets, args, fasta_d):
     APE, fractions, background = find_RUST_ratios(df3, "final", fasta_d, args, A_offset="final")
 
     final_kl = save_KL(fractions=fractions, background=background, this_type="final", offset="final", args=args,
-                       save=args.save_KL)
+                       save=args.save_stats)
 
     return final_kl
 
@@ -1407,6 +1467,7 @@ def plot_RUST_enrichments(full_d, best_nt_offsets, pdf_pages):
 
     # First, work out what types we have
     types = best_nt_offsets.keys()
+    types = order_list(types)
 
     # Create a pandas dataframe with all data for this sample
     for this_type in types:
@@ -1425,7 +1486,7 @@ def plot_RUST_enrichments(full_d, best_nt_offsets, pdf_pages):
         this_df = this_df[~this_df["codon"].isin(["TAG", "TAA", "TGA"])]
 
         # create plot
-        p = sns.heatmap(this_df.drop("codon", axis=1), cbar=False, vmin=-3, vmax=3, cmap="bwr", label='small')
+        p = sns.heatmap(this_df.drop("codon", axis=1), cbar=False, vmin=-2, vmax=2, cmap="bwr", label='small')
         p.set_yticks(range(len(this_df)))
         p.set_yticklabels(this_df["codon"], size=5)
 
@@ -1699,8 +1760,6 @@ def main():
                              "Length is measured in nt. Default = -40")
     parser.add_argument("-ig", "--ignore", required=False, default="None", type=str,
                         help="Read types to ignore. Can add multiple with a colon separator. Eg -ig 27_2:23_1.")
-    parser.add_argument("-wr", "--write_rust", required=False, action="store_true",
-                        help="Write a csv of rust values for each read ")
     parser.add_argument("-of", "--offset", required=False, type=str, default="None",
                         help="If read types are specified with -rt, one can also specify a specific "
                              "offset from the 3' end of the read. "
@@ -1776,8 +1835,6 @@ def main():
     parser.add_argument("--orf_file", type=str, default="none_provided", help="An orf csv generated using the "
                                                                               "--save_orfs function. Reduces run "
                                                                               "time.")
-    parser.add_argument("-skl", "--save_KL", action="store_true", default=False,
-                        help="Save a csv of the KL divergence at each position")
     parser.add_argument("--ambiguity", type=float, default=0.9,
                         help="Much much better the best r value must be compared to the second best for an offset "
                              "to be confidently assigned. Default = 0.9, i.e. the r value of second best offset must "
@@ -1804,13 +1861,12 @@ def main():
     fasta_d = read_fasta(args.transcript_fasta)
     info = read_info_file(args.info)
 
-    if args.save_KL:
+    if args.save_stats:
         with open(args.output_directory + args.output + '.kl_div.csv', 'w') as file:
             file.write(','.join(['read_type', 'offset', 'kl']) + '\n')
 
-
     # read bed/bam file
-    df, total_counts, total_counts_cds = generate_bed_df(args,
+    df, total_counts, total_counts_cds, ss_df = generate_bed_df(args,
                                                          info,
                                                          fasta_d)  # if a csv of files is provided, will read the biggest
     # Generate a reference, or read it in
@@ -1843,7 +1899,7 @@ def main():
         # Convert bed file to single codons
         convert_bed_to_single_codon_res(args=args, df=df, fasta_d=fasta_d, ref_APEs=ref_APEs,
                                         precalculated_offsets=precalculated_offsets, total_counts_cds=total_counts_cds,
-                                        info=info, orf_df=orf_df, oof_df=oof_df)
+                                        info=info, orf_df=orf_df, oof_df=oof_df, ss_df=ss_df)
 
 
 if __name__ == "__main__":
